@@ -1,15 +1,5 @@
 //! Keys and other sensitive cryptographic material <span style="color:red">**HAZMAT**</span>.
 //!
-//! This module provides both originating keys ([`RootKey`]) and derived keys ([`DerivedKey`]) as well
-//! as nonce types and other hazardous material.
-//!
-//! Originating keys ([`RootKey`]) are randomly generated, and have methods for their usage in
-//! generating derived keys, as well methods for encrypting the key with a password for safe at-rest
-//! storage.
-//!
-//! Derived keys ([`DerivedKey`]) are generated via a KDF over the originating key's entropy pool and a
-//! context string incorporating both a random nonce and a namespace.
-//!
 //! # <span style="color:red">**DANGER**</span>
 //!
 //! This module deals in low level cryptographic details. It is advisable to not deal with this module
@@ -21,13 +11,15 @@ use chacha20::{
     cipher::{generic_array::GenericArray, NewCipher, StreamCipher},
     XChaCha20, XNonce,
 };
-use rand::Rng;
+use rand::{Rng, RngCore};
 use serde::{Deserialize, Serialize};
 use snafu::{ensure, ResultExt};
 use zeroize::{Zeroize, Zeroizing};
 
-use crate::crypto::types::{CipherText, ClearText};
-use crate::error::{Argon2Failure, BadHMAC, Error};
+use crate::{
+    crypto::types::{CipherText, ClearText, ConstArray},
+    error::{Argon2Failure, BackendError, BadHMAC},
+};
 
 /// Allows access to the subkeys of a key-like structure
 pub trait Key {
@@ -44,14 +36,14 @@ pub trait Key {
 /// Nonces are intended to be always randomly generated, and there is intentionally no API for
 /// reconstructing a nonce.
 #[derive(Debug, Hash, Clone, PartialEq, Eq, PartialOrd, Ord, Serialize, Deserialize)]
-pub struct Nonce(pub(crate) [u8; 24]);
+pub struct Nonce(pub(crate) ConstArray<24>);
 
 impl Nonce {
     /// Generates a new, random nonce
     pub fn random() -> Self {
         let mut data = [0_u8; 24];
         rand::thread_rng().fill(&mut data[..]);
-        Self(data)
+        Self(data.into())
     }
     /// Gets a view of the nonce as an `XChaCha20` [`XNonce`]
     pub fn nonce(&self) -> &XNonce {
@@ -77,14 +69,11 @@ impl Nonce {
 #[zeroize(drop)]
 pub struct RootKey {
     /// The root cipher key, used for encrypting headers
-    encryption: [u8; 32],
+    encryption: ConstArray<32>,
     /// The root HMAC key, used for validating headers
-    hmac: [u8; 32],
+    hmac: ConstArray<32>,
     /// Random data used to derive encryption keys
-    ///
-    /// This is an array of array's to more easily work around serde not having support for arrays longer
-    /// than 32 bytes
-    entropy: [[u8; 32]; 8],
+    entropy: ConstArray<256>,
 }
 
 impl RootKey {
@@ -103,9 +92,7 @@ impl RootKey {
         // Fill keys
         rand.fill(&mut ret.encryption[..]);
         rand.fill(&mut ret.hmac[..]);
-        for i in 0..8 {
-            rand.fill(&mut ret.entropy[i]);
-        }
+        rand.fill(&mut ret.entropy[..]);
 
         ret
     }
@@ -120,9 +107,9 @@ impl RootKey {
     /// in plaintext would be appropriate.
     pub fn null() -> Self {
         RootKey {
-            encryption: [0_u8; 32],
-            hmac: [0_u8; 32],
-            entropy: [[0_u8; 32]; 8],
+            encryption: [0_u8; 32].into(),
+            hmac: [0_u8; 32].into(),
+            entropy: [0_u8; 256].into(),
         }
     }
     /// Encrypts this key, producing a [`EncryptedRootKey`], with the provided password.
@@ -137,7 +124,7 @@ impl RootKey {
     ///
     /// Will return:
     ///   * `Error::Argon2Failure` if the argon2 key derivation fails
-    pub fn encrypt(&self, password: &[u8]) -> Result<EncryptedRootKey, Error> {
+    pub fn encrypt(&self, password: &[u8]) -> Result<EncryptedRootKey, BackendError> {
         // Generate a random salt
         let mut salt = [0_u8; 32];
         rand::thread_rng().fill(&mut salt[..]);
@@ -162,18 +149,18 @@ impl RootKey {
         let mut chacha = XChaCha20::new(encryption_key, nonce.nonce());
         chacha.apply_keystream(&mut serial[..]);
         // Calculate the HMAC
-        let hmac = blake3::keyed_hash(&hmac_key, &serial);
+        let hmac: [u8; 32] = blake3::keyed_hash(&hmac_key, &serial).into();
         Ok(EncryptedRootKey {
             nonce,
             hmac: hmac.into(),
-            salt,
+            salt: salt.into(),
             payload: serial,
         })
     }
     /// Creates a [`DerivedKey`] from this [`RootKey`] using the provided namespace as part of the
     /// context string
     ///
-    /// This method will generate, using a CSPRNG a random, 32 character, alphanumeric nonce (~192 bits of
+    /// This method will generate, using a CSPRNG a random, 32 character, hexadecimal nonce (~128 bits of
     /// entropy) to include in the context string, which will then be combined with the provided namespace
     /// and fed into Blake3's key derivation mode. Blake3 is used to derive 64 bytes of key material, the
     /// first 32 bytes of which are used as an encryption key, and the last 32 bytes of which are used as an
@@ -183,15 +170,62 @@ impl RootKey {
     /// the usage of an individual [`DerivedKey`] as much as possible, to limit the fallout of any
     /// accidental/unintentional nonce reuses.
     pub fn derive(&self, namespace: &str) -> DerivedKey {
-        // Get 32 alphanumerics for the nonce
-        let mut nonce = Zeroizing::new(String::new());
-        let mut rand = rand::thread_rng();
-        while nonce.len() < 32 {
-            let ch: char = rand.gen();
-            if ch.is_ascii_alphanumeric() {
-                nonce.push(ch);
-            }
+        // Do this manually for speed.
+        let mut nonce_array = [0_u8; 32];
+        let mut nonce_bytes = [0_u8; 16];
+        rand::thread_rng().fill_bytes(&mut nonce_bytes[..]);
+        for (index, byte) in nonce_bytes.into_iter().enumerate() {
+            // Do the upper nibble
+            let upper_nibble = match byte & 0xF0_u8 {
+                0x00 => 0x30_u8,
+                0x10 => 0x31_u8,
+                0x20 => 0x32_u8,
+                0x30 => 0x33_u8,
+                0x40 => 0x34_u8,
+                0x50 => 0x35_u8,
+                0x60 => 0x36_u8,
+                0x70 => 0x37_u8,
+                0x80 => 0x38_u8,
+                0x90 => 0x39_u8,
+                0xA0 => 0x41_u8,
+                0xB0 => 0x42_u8,
+                0xC0 => 0x43_u8,
+                0xD0 => 0x44_u8,
+                0xE0 => 0x45_u8,
+                0xF0 => 0x46_u8,
+                _ => unreachable!(),
+            };
+            // And the lower nibble
+            let lower_nibble = match byte & 0x0F_u8 {
+                0x00 => 0x30_u8,
+                0x01 => 0x31_u8,
+                0x02 => 0x32_u8,
+                0x03 => 0x33_u8,
+                0x04 => 0x34_u8,
+                0x05 => 0x35_u8,
+                0x06 => 0x36_u8,
+                0x07 => 0x37_u8,
+                0x08 => 0x38_u8,
+                0x09 => 0x39_u8,
+                0x0A => 0x41_u8,
+                0x0B => 0x42_u8,
+                0x0C => 0x43_u8,
+                0x0D => 0x44_u8,
+                0x0E => 0x45_u8,
+                0x0F => 0x46_u8,
+                _ => unreachable!(),
+            };
+            // Set the bytes
+            nonce_array[index * 2] = upper_nibble;
+            nonce_array[index * 2 + 1] = lower_nibble;
         }
+        // We are only generating ascii chars, so we can mark the failure as unreachable
+        let nonce = if let Ok(nonce) = std::str::from_utf8(&nonce_array[..]) {
+            nonce
+        } else {
+            unreachable!()
+        };
+
         // Setup the context string
         let context_string = format!("snapper-box nonce: {} namespace: {}", &*nonce, namespace);
         self.derive_with_context(context_string)
@@ -207,13 +241,11 @@ impl RootKey {
         // Setup the hasher
         let mut hasher = Hasher::new_derive_key(&context_string);
         // Load in the entropy
-        for item in &self.entropy {
-            hasher.update(item);
-        }
+        hasher.update(&self.entropy[..]);
         // Make the final derived key
         let mut ret = DerivedKey {
-            encryption: [0_u8; 32],
-            hmac: [0_u8; 32],
+            encryption: [0_u8; 32].into(),
+            hmac: [0_u8; 32].into(),
             context_string,
         };
         // Load in the keys, and return
@@ -231,7 +263,7 @@ impl Key for RootKey {
     }
     /// Provides the hmac key as a reference to the underlying array
     fn hmac_key(&self) -> &[u8; 32] {
-        &self.hmac
+        self.hmac.as_ref()
     }
 }
 
@@ -243,10 +275,11 @@ pub struct EncryptedRootKey {
     /// The nonce used for the encryption
     nonce: Nonce,
     /// The HMAC tag
-    hmac: [u8; 32],
+    hmac: ConstArray<32>,
     /// The salt used for argon2
-    salt: [u8; 32],
+    salt: ConstArray<32>,
     /// The encrypted payload
+    #[serde(with = "serde_bytes")]
     payload: Vec<u8>,
 }
 
@@ -259,7 +292,7 @@ impl EncryptedRootKey {
     ///   * `Error::Argon2Failure` if the argon2 key derivation fails
     ///   * `Error::BadHmac` if the hmac verification failed, either due to incorrect password or corruption
     ///   * `Error::KeyDeserialization` if the key fails to deserialize, which really shouldn't happen
-    pub fn decrypt(&self, password: &[u8]) -> Result<RootKey, Error> {
+    pub fn decrypt(&self, password: &[u8]) -> Result<RootKey, BackendError> {
         // Prepare the argon2 instance
         let argon = Argon2::default();
         // Prepare the output buffer
@@ -274,7 +307,7 @@ impl EncryptedRootKey {
         hmac_key.copy_from_slice(&argon_output[32..]);
         // Verify the HMAC
         let hmac = blake3::keyed_hash(&hmac_key, &self.payload);
-        ensure!(hmac.eq(&self.hmac), BadHMAC);
+        ensure!(hmac.eq(&*self.hmac), BadHMAC);
         // Decrypt the data
         let mut data = Zeroizing::new(self.payload.clone());
         let mut chacha = XChaCha20::new(encryption_key, self.nonce.nonce());
@@ -284,7 +317,7 @@ impl EncryptedRootKey {
             Ok(x) => Ok(x),
             Err(_) => {
                 // Do not preserve the serde error, this may leak secrets into logs
-                Err(Error::KeyDeserialization)
+                Err(BackendError::KeyDeserialization)
             }
         }
     }
@@ -301,9 +334,9 @@ impl EncryptedRootKey {
 #[zeroize(drop)]
 pub struct DerivedKey {
     /// The encryption key
-    encryption: [u8; 32],
+    encryption: ConstArray<32>,
     /// The root HMAC key, used for validating headers
-    hmac: [u8; 32],
+    hmac: ConstArray<32>,
     /// The context string used to produce this key
     context_string: String,
 }
@@ -315,7 +348,7 @@ impl Key for DerivedKey {
     }
     /// Provides the hmac key
     fn hmac_key(&self) -> &[u8; 32] {
-        &self.hmac
+        self.hmac.as_ref()
     }
 }
 
@@ -330,7 +363,10 @@ impl DerivedKey {
     /// # Errors
     ///
     /// Will error if the encryption or serialization fail
-    pub fn encrypt(&self, root_key: &RootKey) -> Result<EncryptedDerivedKey<'static>, Error> {
+    pub fn encrypt(
+        &self,
+        root_key: &RootKey,
+    ) -> Result<EncryptedDerivedKey<'static>, BackendError> {
         // First get a cleartext containing ourself
         let cleartext = ClearText::new(self)?;
         // Then encrypt it with the root key, without compression
@@ -356,7 +392,7 @@ impl EncryptedDerivedKey<'_> {
     /// # Errors
     ///
     /// Will error if the decryption or deserialization fails
-    pub fn decrypt(&self, root_key: &RootKey) -> Result<DerivedKey, Error> {
+    pub fn decrypt(&self, root_key: &RootKey) -> Result<DerivedKey, BackendError> {
         // Attempt to get the cleartext
         let cleartext = self.encrypted_key.decrypt(root_key)?;
         // Attempt to deserialize it
@@ -375,21 +411,17 @@ mod tests {
         #[test]
         fn null_is_zeros() {
             let key = RootKey::null();
-            assert_eq!(key.encryption, [0_u8; 32]);
-            assert_eq!(key.hmac, [0_u8; 32]);
-            for i in 0..8 {
-                assert_eq!(key.entropy[i], [0_u8; 32]);
-            }
+            assert_eq!(key.encryption, [0_u8; 32].into());
+            assert_eq!(key.hmac, [0_u8; 32].into());
+            assert_eq!(key.entropy, [0_u8; 256].into());
         }
         /// Make sure randomly generated key has no zero segments
         #[test]
         fn random_is_not_zeros() {
             let key = RootKey::random();
-            assert_ne!(key.encryption, [0_u8; 32]);
-            assert_ne!(key.hmac, [0_u8; 32]);
-            for i in 0..8 {
-                assert_ne!(key.entropy[i], [0_u8; 32]);
-            }
+            assert_ne!(key.encryption, [0_u8; 32].into());
+            assert_ne!(key.hmac, [0_u8; 32].into());
+            assert_ne!(key.entropy, [0_u8; 256].into());
         }
     }
     /// Unit tests for [`Nonce`]
@@ -399,7 +431,7 @@ mod tests {
         #[test]
         fn non_zero() {
             let nonce = Nonce::random();
-            assert_ne!(nonce.0, [0_u8; 24]);
+            assert_ne!(nonce.0, [0_u8; 24].into());
         }
     }
     /// Unit tests for [`EncryptedRootKey`]
@@ -437,7 +469,7 @@ mod tests {
             let decrypted = encrypted.decrypt(password);
             match decrypted {
                 Ok(_) => panic!("Somehow decrypted corrupted data"),
-                Err(e) => assert!(matches!(e, Error::BadHMAC)),
+                Err(e) => assert!(matches!(e, BackendError::BadHMAC)),
             }
         }
     }
@@ -450,8 +482,8 @@ mod tests {
         fn not_zero() {
             let root_key = RootKey::random();
             let derived_key = root_key.derive("namespace");
-            assert_ne!(derived_key.encryption, [0_u8; 32]);
-            assert_ne!(derived_key.hmac, [0_u8; 32]);
+            assert_ne!(derived_key.encryption, [0_u8; 32].into());
+            assert_ne!(derived_key.hmac, [0_u8; 32].into());
             assert_ne!(derived_key.encryption, derived_key.hmac);
         }
         /// Ensure that repeated calls to derive key are different
