@@ -12,22 +12,22 @@ use std::{
 };
 
 use serde::{de::DeserializeOwned, Deserialize, Serialize};
+use serde_bytes::ByteBuf;
 use snafu::ResultExt;
 use tracing::{debug, instrument, warn};
 
 use crate::{
     crypto::{CipherText, ClearText, Key},
-    error::{EntryIO, Error},
+    error::{BackendError, EntryIO},
     file::segment::Segment,
 };
-
-pub mod segment;
+mod segment;
 
 /// Internal, data containing struct used for serializing log entries
 #[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
 struct Entry {
     /// Items being stored
-    items: HashMap<Vec<u8>, Vec<u8>>,
+    items: HashMap<ByteBuf, ByteBuf>,
 }
 
 /// LSM Abstraction over something [`File`] like (i.e. implementing [`Read`], [`Write`], and [`Seek`]).
@@ -73,7 +73,7 @@ where
         compression: Option<i32>,
         key: Arc<K>,
         max_cache_entries: Option<usize>,
-    ) -> Result<Self, Error> {
+    ) -> Result<Self, BackendError> {
         // Seek to start of file
         file.seek(SeekFrom::Start(0)).context(EntryIO)?;
         // Start trying to read segments
@@ -85,7 +85,7 @@ where
                     let cleartext = x.decrypt(&*key)?;
                     let entry: Entry = cleartext.deserialize()?;
                     for (k, _) in entry.items {
-                        offsets.insert(k, offset);
+                        offsets.insert(k.to_vec(), offset);
                     }
                 }
                 Err(e) => {
@@ -111,17 +111,17 @@ where
     /// # Errors
     ///
     /// Will return an error if any IO error occurs, or if the value fails to deserialize.
-    pub fn get<C, V>(&mut self, key: &C) -> Result<Option<V>, Error>
+    pub fn get<C, V>(&mut self, key: &C) -> Result<Option<V>, BackendError>
     where
         C: Serialize,
         V: DeserializeOwned,
     {
         // Attempt to serialize the key
         let key = match serde_cbor::to_vec(key) {
-            Ok(k) => k,
+            Ok(k) => ByteBuf::from(k),
             // Intentionally hide the underlying serde error, to avoid leaking sensitive data into
             // the logs
-            Err(_) => return Err(Error::ItemSerialization),
+            Err(_) => return Err(BackendError::ItemSerialization),
         };
         // Look up the key in the cache, return from it if possible
         if let Some(cache) = self.cache.as_ref() {
@@ -130,12 +130,12 @@ where
                     Ok(v) => return Ok(Some(v)),
                     // Intentionally hide the underlying serde error, to avoid leaking sensitive
                     // data into the logs
-                    Err(_) => return Err(Error::ItemDeserialization),
+                    Err(_) => return Err(BackendError::ItemDeserialization),
                 }
             }
         }
         // Look up the key in the offsets table
-        if let Some(offset) = self.offsets.get(&key) {
+        if let Some(offset) = self.offsets.get(&*key) {
             // Find the segment
             self.file.seek(SeekFrom::Start(*offset)).context(EntryIO)?;
             let segment = Segment::read_owned(&mut self.file)?;
@@ -148,7 +148,7 @@ where
                     Ok(v) => Ok(Some(v)),
                     // Intentionally hide the underlying serde error, to avoid leaking sensitive
                     // data into the logs
-                    Err(_) => Err(Error::ItemDeserialization),
+                    Err(_) => Err(BackendError::ItemDeserialization),
                 }
             } else {
                 warn!("Offsets table contained a offset fora nonexistent value");
@@ -168,7 +168,7 @@ where
     ///   * Will return an error if either the key or value fail to serialize
     ///   * Will bubble up any IO errors that occur, if a flush occurs
     #[instrument(skip(self, key, value))]
-    pub fn insert<C, V>(&mut self, key: &C, value: &V) -> Result<(), Error>
+    pub fn insert<C, V>(&mut self, key: &C, value: &V) -> Result<(), BackendError>
     where
         C: Serialize,
         V: Serialize,
@@ -179,10 +179,10 @@ where
         });
         // Serialize the values
         // Intentionally hide the serde errors, to avoid leaking sensitive information
-        let key = serde_cbor::to_vec(key).map_err(|_| Error::ItemSerialization)?;
-        let value = serde_cbor::to_vec(value).map_err(|_| Error::ItemSerialization)?;
+        let key = serde_cbor::to_vec(key).map_err(|_| BackendError::ItemSerialization)?;
+        let value = serde_cbor::to_vec(value).map_err(|_| BackendError::ItemSerialization)?;
         // Insert it into the cache
-        cache.items.insert(key, value);
+        cache.items.insert(ByteBuf::from(key), ByteBuf::from(value));
         let length = cache.items.len();
         self.cache = Some(cache);
         // Flush if needed
@@ -198,14 +198,14 @@ where
     /// # Errors
     ///
     /// Will bubble up any IO errors
-    pub fn flush(&mut self) -> Result<(), Error> {
+    pub fn flush(&mut self) -> Result<(), BackendError> {
         // Get the end of the file
         let offset = self.file.seek(SeekFrom::End(0)).context(EntryIO)?;
         // Take the cache
         if let Some(cache) = self.cache.take() {
             // Update the offset table
             for k in cache.items.keys() {
-                self.offsets.insert(k.clone(), offset);
+                self.offsets.insert((&*k).to_vec(), offset);
             }
             // Serialize it and encrypt it
             let plaintext = ClearText::new(&cache)?;
@@ -228,7 +228,7 @@ where
     /// # Errors
     ///
     /// Will bubble up any error that occurs during a `get` operation
-    pub fn to_hashmap<C, V>(&mut self) -> Result<HashMap<C, V>, Error>
+    pub fn to_hashmap<C, V>(&mut self) -> Result<HashMap<C, V>, BackendError>
     where
         C: DeserializeOwned + Serialize + Hash + Eq,
         V: DeserializeOwned,
@@ -238,27 +238,87 @@ where
         // Insert from storage
         for key in keys {
             // Deserialize the key
-            let key: C = serde_cbor::from_slice(&key).map_err(|_| Error::ItemDeserialization)?;
+            let key: C =
+                serde_cbor::from_slice(&key).map_err(|_| BackendError::ItemDeserialization)?;
             // get the value
-            let value = self.get(&key)?.ok_or(Error::InvalidLsmState)?;
+            let value = self.get(&key)?.ok_or(BackendError::InvalidLsmState)?;
             ret.insert(key, value);
         }
         // Insert from cache
         if let Some(cache) = self.cache.as_ref() {
-            let keys: Vec<Vec<u8>> = cache.items.keys().cloned().collect();
+            let keys: Vec<ByteBuf> = cache.items.keys().cloned().collect();
             for key in keys {
                 // Deserialize the key
                 let key_deser: C =
-                    serde_cbor::from_slice(&key).map_err(|_| Error::ItemDeserialization)?;
+                    serde_cbor::from_slice(&key).map_err(|_| BackendError::ItemDeserialization)?;
                 // get the value
-                let value = cache.items.get(&key).ok_or(Error::InvalidLsmState)?;
+                let value = cache.items.get(&key).ok_or(BackendError::InvalidLsmState)?;
                 let value: V =
-                    serde_cbor::from_slice(value).map_err(|_| Error::ItemDeserialization)?;
+                    serde_cbor::from_slice(value).map_err(|_| BackendError::ItemDeserialization)?;
                 ret.insert(key_deser, value);
             }
         }
         Ok(ret)
     }
+    /// Converts the current contents of the store to a [`Vec`] of pairs.
+    ///
+    /// Iterates through this `LsmFIle`'s keys, and insert them all into the returned [`Vec`]]. The most
+    /// up to date value will be used for each key.
+    ///
+    /// # Errors
+    ///
+    /// Will bubble up any error that occurs during a `get` operation
+    pub fn to_pairs<C, V>(&mut self) -> Result<Vec<(C, V)>, BackendError>
+    where
+        C: DeserializeOwned + Serialize + Eq + Clone,
+        V: DeserializeOwned + Clone,
+    {
+        let mut ret: Vec<(C, V)> = Vec::new();
+        let keys: Vec<Vec<u8>> = self.offsets.keys().cloned().collect();
+        // Insert from storage
+        for key in keys {
+            // Deserialize the key
+            let key: C =
+                serde_cbor::from_slice(&key).map_err(|_| BackendError::ItemDeserialization)?;
+            // get the value
+            let value = self.get(&key)?.ok_or(BackendError::InvalidLsmState)?;
+            if let Some(index) =
+                ret.iter()
+                    .cloned()
+                    .enumerate()
+                    .find_map(|(x, (k, _))| if k == key { Some(x) } else { None })
+            {
+                ret[index] = (key, value);
+            } else {
+                ret.push((key, value));
+            }
+        }
+        // Insert from cache
+        if let Some(cache) = self.cache.as_ref() {
+            let keys: Vec<ByteBuf> = cache.items.keys().cloned().collect();
+            for key in keys {
+                // Deserialize the key
+                let key_deser: C =
+                    serde_cbor::from_slice(&key).map_err(|_| BackendError::ItemDeserialization)?;
+                // get the value
+                let value = cache.items.get(&key).ok_or(BackendError::InvalidLsmState)?;
+                let value: V =
+                    serde_cbor::from_slice(value).map_err(|_| BackendError::ItemDeserialization)?;
+                if let Some(index) = ret.iter().cloned().enumerate().find_map(|(x, (k, _))| {
+                    if k == key_deser {
+                        Some(x)
+                    } else {
+                        None
+                    }
+                }) {
+                    ret[index] = (key_deser, value);
+                } else {
+                    ret.push((key_deser, value));
+                }
+            }
+        }
+        Ok(ret)
+    }
     /// Consumes self and returns the inner [`File`] like object
     pub fn into_inner(self) -> T {
         self.file
@@ -283,7 +343,7 @@ where
         compression: Option<i32>,
         key: Arc<K>,
         max_cache_entries: Option<usize>,
-    ) -> Result<Self, Error> {
+    ) -> Result<Self, BackendError> {
         // open the file read/write
         let file = OpenOptions::new()
             .read(true)
@@ -307,7 +367,7 @@ where
         compression: Option<i32>,
         key: Arc<K>,
         max_cache_entries: Option<usize>,
-    ) -> Result<Self, Error> {
+    ) -> Result<Self, BackendError> {
         // open the file read/write
         let file = OpenOptions::new()
             .read(true)
@@ -329,7 +389,7 @@ mod tests {
     use tempfile::{tempdir, tempfile};
     /// Make sure that `LsmFile` can create and open files
     #[test]
-    fn file_create_open() -> Result<(), Error> {
+    fn file_create_open() -> Result<(), BackendError> {
         let dir = tempdir().expect("Unable to make tempdir");
         let file_path = dir.path().join("test_lsm");
         println!("{:?}", file_path);
